#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "functions.h"
#include "functions_sse.h"
#include "huffman.h"
#include "list.h"

//includes para opencv
#include <cv.h>
#include "highgui.h"

int descompress_AVI (char* name_video_in, int assembler)
{
	/*
	 * propiedades que tenemos que guardar al principio del header para la descompresion
	 * CV_CAP_PROP_FRAME_WIDTH Width of the frames in the video stream.
	 * CV_CAP_PROP_FRAME_HEIGHT Height of the frames in the video stream.
	 * CV_CAP_PROP_FPS Frame rate.
	 * CV_CAP_PROP_FOURCC codigo de video original
	*/
	char name_out_header[strlen(name_video_in) +7 + 1];	// +7 por .header + 1 por null 
	char name_out_frames[strlen(name_video_in) + 4 + 1];	// +4 por .oc2 + 1 por null 
	char name_video_out[strlen(name_video_in) + 7 + 1];	// + 7 por oc2.AVI + 1 por null 
	
	strncpy(name_out_header, name_video_in, sizeof(name_out_header));
	strcat(name_out_header, ".header");
	strncpy(name_out_frames, name_video_in, sizeof(name_out_frames));
	strcat(name_out_frames, ".oc2");
	strncpy(name_video_out, name_video_in, sizeof(name_video_out));
	strcat(name_video_out, "oc2.AVI");
	
	FILE * headers_file;
	FILE * frames_file;
	headers_file = fopen(name_out_header, "rb+");
	frames_file = fopen(name_out_frames, "rb+");
	cvNamedWindow( "w", 1);
	/* asumo que el oc2_main header es el que tiene las cosas del video*/
	
	oc2_main_header_t * oc2_main_header = read_main_header_oc2 (headers_file);
	
	int cant_frames =oc2_main_header->cant_frames ;
	int fourcc = oc2_main_header->fourcc ;
	double fps = oc2_main_header->fps ;
	int width = oc2_main_header->width ; 
	int height = oc2_main_header->height;
	CvSize frame_size = cvSize(width, height);
	//~ printf("cant frames: %i \n", cant_frames);
	CvVideoWriter* video_out = cvCreateVideoWriter(name_video_out, fourcc, fps, frame_size, 0 );
	
	int curr_frame;
	for(curr_frame = 0; curr_frame < cant_frames; curr_frame++)
	{
		int **i_matrix;
		int i;
		i_matrix = (int**) malloc (height * sizeof (int*));    
		for (i = 0; i < height; i++)
			i_matrix[i] = (int*) malloc (width * sizeof (int)); 

		int seek_header, seek_oc2; /* estos seeks son el puntero a donde se tiene que mover en el archivo para descomprimir el frame*/
		descompress_frame(i_matrix, width, height, headers_file, frames_file, seek_header, seek_oc2);
		if (assembler == corre_con_asm) jpeg_dec_asm(i_matrix, width, height);
		if (assembler == corre_con_c) jpeg_dec(i_matrix, width, height);

		//~ jpeg_dec(i_matrix, width, height);
		cvWaitKey(30);
		IplImage* frame = cvCreateImage(frame_size, IPL_DEPTH_8U, 1);
		write_image_ipl (frame, i_matrix);
		cvShowImage("w", frame);

		int write_OK = cvWriteFrame(video_out, frame);
		for (i = 0; i < height; i++)
				free(i_matrix[i]); 
		free(i_matrix);

	}
	
	cvWaitKey(30);
	cvDestroyWindow("w");
	free(oc2_main_header);
	fclose(headers_file);
	fclose(frames_file);
	remove(name_out_frames);
	remove(name_out_header);

	
}


void descompress_frame(int ** i_matrix, int width, int height, FILE * headers_file, FILE * frames_file, int seek_header, int seek_oc2)
{
	oc2_header_t * oc2;
	oc2 = leer_info_oc2(headers_file);  /* aca hay que ver que el puntero en el archivo este bien parado en cada frame */
	nodo_lista_c * lista_c = NULL;
	
	lista_c = leer_codigos(headers_file, oc2->cant_codigos, oc2->offset_tabla);
	
	nodo_arbol_t * arbol_huff = generar_arbol_descomprecion (lista_c, oc2->cant_codigos);
	
	int tam_codif_bytes = (oc2->long_codificacion + (32-oc2->long_codificacion % 32))/8;
	//~ printf("tam_codif_bytes: %i\n", tam_codif_bytes);
	//~ printf("long_codificacion: %i\n", oc2->long_codificacion);
	unsigned int * buffer = malloc(tam_codif_bytes);
	if(buffer == 0) printf("malloc dio null no hay mas memoria amigou");
	unsigned int * buffer_aux = buffer;
	
	int i;
	for (i = 0; i < tam_codif_bytes/4; ++i)
	{
		fread(buffer_aux, sizeof(unsigned int), 1, frames_file);
		buffer_aux++;
	}
	
	
	descomprimir_buffer(oc2->long_codificacion, buffer, i_matrix, width, height, arbol_huff);
	free(buffer);
	free(oc2);
	int size;
	return 0;
}


void crear_video_avi_sin_audio(char* name_video_in)
{
	char name_video_out[strlen(name_video_in) + 16 + 1];	// + 7 por audiolessoc2.AVI + 1 por null 
	
	strncpy(name_video_out, name_video_in, sizeof(name_video_out));
	strcat(name_video_out, "audiolessoc2.AVI");
	
	CvCapture* capture=0;
    IplImage* frame=0;
    capture = cvCaptureFromAVI(name_video_in); // read AVI video    
        
    int cant_frames = (int)cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_COUNT);
	int fourcc = (int)cvGetCaptureProperty(capture, CV_CAP_PROP_FOURCC);
	double fps = cvGetCaptureProperty(capture, CV_CAP_PROP_FPS);
	int width = (int)cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH); 
	int height = (int)cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_HEIGHT);
	
	CvSize frame_size = cvSize(width, height);
	
	CvVideoWriter* video_out = cvCreateVideoWriter(name_video_out, fourcc, fps, frame_size, 0 );

	
	int curr_frame;
	for(curr_frame = 0; curr_frame < cant_frames; curr_frame++)
	{
		frame = cvQueryFrame( capture );
		
        if(!frame) break;
        
        IplImage *im_gray = cvCreateImage(cvGetSize(frame),IPL_DEPTH_8U,1);
		cvCvtColor(frame,im_gray,CV_BGR2GRAY);

		int write_OK = cvWriteFrame(video_out, im_gray);
	}
}

int descompress_image (char* name_image_in, int assembler)
{
	char name_out_header[strlen(name_image_in) +7 + 1];	// +7 por .header + 1 por null 
	char name_out_image[strlen(name_image_in) + 4 + 1];	// +4 por .oc2 + 1 por null 
	char name_image_out[strlen(name_image_in) + 7 + 1];	// + 7 por oc2.AVI + 1 por null 
	
	strncpy(name_out_header, name_image_in, sizeof(name_out_header));
	strcat(name_out_header, ".header");
	strncpy(name_out_image, name_image_in, sizeof(name_out_image));
	strcat(name_out_image, ".oc2");
	strncpy(name_image_out, name_image_in, sizeof(name_image_out));
	strcat(name_image_out, "oc2.bmp");
	
	FILE * header_file;
	FILE * image_file;
	header_file = fopen(name_out_header, "rb+");
	image_file = fopen(name_out_image, "rb+");

	oc2_main_header_t * oc2_main_header = read_main_header_oc2 (header_file);

	int height = oc2_main_header->height;
	int width = oc2_main_header->width;
	printf("Tamanio de la imagen %i %i\n", width, height);
	
	oc2_header_t * oc2;
	oc2 = leer_info_oc2(header_file);  /* aca hay que ver que el puntero en el archivo este bien parado en cada frame */

	int **i_matrix;
	int i;
	i_matrix = (int**) malloc (height * sizeof (int*));    
	for (i = 0; i < height; i++)
		i_matrix[i] = (int*) malloc (width * sizeof (int)); 

	nodo_lista_c * lista_c = NULL;
	
	lista_c = leer_codigos(header_file, oc2->cant_codigos, oc2->offset_tabla);
	
	nodo_arbol_t * arbol_huff = generar_arbol_descomprecion (lista_c, oc2->cant_codigos);

	int tam_codif_bytes = (oc2->long_codificacion + (32-oc2->long_codificacion % 32))/8;
	unsigned int * buffer = malloc(tam_codif_bytes);
	unsigned int * buffer_aux = buffer;

	int cant_bytes_read;

	for (i = 0; i < tam_codif_bytes/4; ++i)
	{
		cant_bytes_read = fread(buffer_aux, sizeof(unsigned int), 1, image_file);
		buffer_aux++;
	}
	
	descomprimir_buffer(oc2->long_codificacion, buffer, i_matrix, width, height, arbol_huff);
	
	free(buffer);
	free(oc2);

	if (assembler == corre_con_asm) jpeg_dec_asm(i_matrix, width, height);
	if (assembler == corre_con_c) jpeg_dec(i_matrix, width, height);

	IplImage* image = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 1);
	write_image_ipl (image, i_matrix);
	
	cvSaveImage(name_image_out, image,"CV_IMWRITE_PXM_BINARY,1");

	//Libero la memoria de la matriz
	for (i = 0; i < height; i++)
			free(i_matrix[i]); 
	free(i_matrix);
	
	fclose(header_file);
	fclose(image_file);	
	remove(name_out_image);
	remove(name_out_header);	
}
